/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 package examples;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;

import edu.uci.isr.yancees.EventInterface;
import edu.uci.isr.yancees.GenericSubscription;
import edu.uci.isr.yancees.ProtocolFacade;
import edu.uci.isr.yancees.SubscriberInterface;
import edu.uci.isr.yancees.SubscriptionFacade;
import edu.uci.isr.yancees.YanceesException;
import edu.uci.isr.yancees.YanceesFacade;
import edu.uci.isr.yancees.core.MessageParsingException;
import edu.uci.isr.yancees.server.plugin.protocol.polling.PollProtocolPlugin;

/**
 * This program connects to the protocol locally.
 *
 * <p>Title: Yancees Notification Server</p>
 * <p>Description: Yet ANother Configurable Extensible Event Service</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: School of Information and Computer Science - University of California, Irvine</p>
 * @author Roberto Silveira Silva Filho
 * @version 1.0
 */
public class ProtocolClient
      implements SubscriberInterface {

   public static void main(String argv[]) {

      if (argv.length != 2) {
         System.err.println("Usage: java ProtocolClient subscriptionMessage.xml configuration.xml");
         System.out.println("Note that the subscription needs to use pull notification");
         System.exit(1);
      }

      YanceesFacade mainAPI = YanceesFacade.getInstance();
      try {
         mainAPI.bootstrap(new File(argv[1]));
      }catch (MessageParsingException ex) {
         System.out.println(ex);
      }
      SubscriptionFacade subAPI = mainAPI.getSubscriberAPI();
      GenericSubscription msg = null;
		try {
			msg = new GenericSubscription(new File(argv[0]));
		} catch (IOException e2) {
			e2.printStackTrace();
		}

      // This class will listen to the events coming from the subscription.
      ProtocolClient myInstance = new ProtocolClient();

      System.out.println("ProtocolClient: Placing subscription...");
      try {
         subAPI.subscribe(msg, myInstance);
      } catch (YanceesException ex) {
         System.out.println(ex);
      }

      ProtocolFacade protAPI = mainAPI.getProtocolAPI();
      PollProtocolPlugin poolPlugin = null;
      try {
         poolPlugin = (PollProtocolPlugin) protAPI.connectToNewProtocol("poll", myInstance);
      } catch (YanceesException e1) {
         // TODO Auto-generated catch block
         e1.printStackTrace();
      }
      
      try {
         poolPlugin.startPeriodicPolling(1000);
      } catch (RemoteException e) {
         // TODO Auto-generated catch block
         e.printStackTrace();
      }
      
      if (poolPlugin != null) {
         System.out.println("ProtocolClient: Poll session created, waiting for messages!");
      }

      while (true) {
         try {
            Thread.sleep(500);
         } catch (InterruptedException ex) {
            System.out.println(ex);
         }
      }

   } // main

//------------------------- Subscription notification handlers -----------------

   /**
    * sends an <code>Event</code> to this <code>ProtocolClient</code>
    *
    * @param n Event passed to the ProtocolClient
    *
    **/
   public void notify(EventInterface evt) {
      System.out.println("ProtocolClient: got Event: \n\n" + evt.toString());
   }

   /**
    * sends a sequence of <code>Event</code> evt to this
    * <code>ProtocolClient</code>
    *
    * @param s sequence of Events passed to the ProtocolClient
    *
    **/
   public void notify(EventInterface[] evtList) {
      for (int i = 0; i < evtList.length; i++) {
         System.out.println("ProtocolClient: got Event: \n\n" + evtList[i].toString());
      }

   }


}

