/*
 * Copyright (c) 2002, Regents of the University of California. All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL WEBCOLLAB.COM OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 package examples;

import java.io.File;
import java.io.IOException;

import edu.uci.isr.yancees.GenericEvent;
import edu.uci.isr.yancees.YanceesException;
import edu.uci.isr.yancees.client.rmi.YanceesRMIClient;

public class RMIPublisher {

   public static void main(String argv[]) {

      int waitTime = 500;

      if (argv.length != 2) {
         System.err.println("Usage: java examples.RMIPublisher hostname filename.xml");
         System.exit(1);
      }
      
      String hostname = argv[0];
      String fileName = argv[1];

		YanceesRMIClient yancees = null;
		try {
			yancees = new YanceesRMIClient(hostname);
		} catch (YanceesException e) {
			System.out.println(e);
			e.printStackTrace();
		}

      GenericEvent evt = null;
      try {
         evt = new GenericEvent(new File(fileName));
      } catch (IOException ex) {
         System.out.println(ex);
         System.exit(1);
      }

		System.out.println("Publishing event: ");
		System.out.println(evt.getXMLTextContent());

      try {
         yancees.publish(evt);
      } catch (YanceesException ex) {
         System.out.println("RMIPublisher: Publishing error.");
         System.out.println(ex);
         System.exit(1);
      }


     // wait some time...
     // we need to give time to siena to process the publishing of the event
     // before we abruptly disconnect from it.
     
     try {
        System.out.println("RMIPublisher: Waiting for: "+waitTime+" ms...");
        java.lang.Thread.sleep(waitTime);
     }catch (InterruptedException ex) {
        System.out.println(ex);
     }
	 

     // and finally exit.
     System.exit(0);


   }


}