/*
 * Copyright (c) 2003, Regents of the University of California.
 * All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE INSTITUTE FOR SOFTWARE RESEARCH AT THE UNIVERSITY
 * OF CALIFORNIA, IRVINE, OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 
package examples.gui;

import java.io.File;
import java.io.IOException;

import edu.uci.isr.yancees.GenericEvent;
import edu.uci.isr.yancees.YanceesException;
import edu.uci.isr.yancees.client.rmi.YanceesRMIClient;


/**
 * @author rsilvafi
 * class created at May 18, 2004
 * 
 */
public class GUIPublisher {

   YanceesRMIClient yancees = null;
   String hostname = "localhost";
   String fileName = null;
   GUIPublisherFrame myFrame = null;
   
   public static void main(String argv[]) {


      if (argv.length != 0) {
         System.err.println("Usage: java examples.gui.GUIPublisher");
         System.exit(1);
      }

      // Create an instance of this class
      GUIPublisher myInstance;
      myInstance = new GUIPublisher();

      //System.out.println("RMISubscriber: Waiting for notifications...");
      /*
      while (true) {
         try {
            Thread.sleep(500);
         } catch (InterruptedException ex) {
            System.out.println(ex);
         }
      }
      */

   } // main
   
   // constructor
   public GUIPublisher() {
      super();
      myFrame = new GUIPublisherFrame("Publisher", this);
     
      // Make window vissible
      myFrame.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
      myFrame.setSize(500, 400);
      myFrame.setVisible(true);
      
   }
   
   
   public void connectToYancees (String hostname) {
      try {
         yancees = new YanceesRMIClient(hostname);
         
      } catch (YanceesException ex) {
         System.out.println(ex);
      }
   }
   
   public boolean isConnected() {
      return yancees != null;
   }
   
   public void disconnect() {
      yancees = null;
   }

   public void publishString(String event) throws YanceesException {
      GenericEvent evt = null;
      
      System.out.println("GUIPublisher: parsing the event...");
     
      evt = new GenericEvent(event); 

      System.out.println("GUIPublisher: publishing the event...");
      yancees.publish(evt);
   }
   
   public void publishFile(File file) throws YanceesException {   
      
      GenericEvent evt = null;
      
      System.out.println("GUIPublisher: parsing the event...");
      try {
         evt = new GenericEvent(file);
      } catch (IOException ex) {
         System.out.println(ex);
         System.out.println(ex.getStackTrace());
         System.exit(1);
      }

      System.out.println("GUIPublisher: publishing the event...");
      yancees.publish(evt);       
   }
   
   public void publishFileName(String filename) throws YanceesException {

      File file = new File(filename);
      publishFile(file);    
   }
   
 
}
