/*
 * Copyright (c) 2003, Regents of the University of California.
 * All rights reserved.
 * 
 * ===================================================================
 * The Apache Software License, Version 1.1
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        the Institute for Software Research at  
 *        University of California, Irvine"
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 * 
 * 4. The names "YANCEES", "Institute for Software Research" and
 *    "University of California" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    rsilvafi@ics.uci.edu or redmiles@ics.uci.edu.
 * 
 * 5. Products derived from this software may not be called
 *    "YANCEES", nor may "YANCEES" appear in their name, without
 *    prior written permission of the University of California Regents.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE INSTITUTE FOR SOFTWARE RESEARCH AT THE UNIVERSITY
 * OF CALIFORNIA, IRVINE, OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
 
package edu.uci.isr.yancees.plugin;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;


/**
 * @author rsilvafi
 * class created at May 3, 2004
 * 
 * Keeps a list of all the factories (representing installed plug-ins) of the
 * system.
 * 
 */
public abstract class AbstractPluginManager implements PluginManagerInterface {

   protected HashMap factoriesMap; // map from tags to factories
   protected HashMap tagsMap; // reverse map from factories to tags
   protected PluginFactoryInterface genericFactory;
   
   public AbstractPluginManager() {
      factoriesMap = new HashMap();
      tagsMap = new HashMap();
   }
   
   /* (non-Javadoc)
    * @see edu.uci.isr.yancees.plugin.PluginManagerInterface#addFactory(edu.uci.isr.yancees.plugin.PluginFactoryInterface)
    */
   public void addFactory(PluginFactoryInterface factory) {
      String[] tags = factory.getTags();
      for (int i = 0; i < tags.length; i++) {
         factoriesMap.put(tags[i], factory);
      }
      tagsMap.put(factory, tags);

   }

   /* (non-Javadoc)
    * @see edu.uci.isr.yancees.plugin.PluginManagerInterface#addUnknownTagFactory(edu.uci.isr.yancees.plugin.PluginFactoryInterface)
    */
   public void addUnknownTagFactory(PluginFactoryInterface factory) {
      genericFactory = factory;
   }

   /* (non-Javadoc)
    * @see edu.uci.isr.yancees.plugin.PluginManagerInterface#removeFactory(edu.uci.isr.yancees.plugin.PluginFactoryInterface)
    */
   public void removeFactory(PluginFactoryInterface factory) {
      String[] tags = (String[]) tagsMap.get(factory);
      tagsMap.remove(factory);
      if (tags != null) {
         for (int i = 0; i < tags.length; i++) {
            factoriesMap.remove(tags[i]);
         }

      }

   }

   /* (non-Javadoc)
    * @see edu.uci.isr.yancees.plugin.PluginManagerInterface#getFactoriesList()
    */
   public PluginFactoryInterface[] getFactoriesList() {
      PluginFactoryInterface[] factories;
      Set s = tagsMap.keySet();
      factories = new PluginFactoryInterface[s.size()];
      if (!s.isEmpty()) {
         int j = 0;
         for (Iterator i = s.iterator(); i.hasNext(); ) {
            Object item = i.next();
            factories[j] = (PluginFactoryInterface) item;
            j++;
         }
      }

      return factories;
   }

   /* (non-Javadoc)
    * @see edu.uci.isr.yancees.plugin.PluginManagerInterface#getRegisteredTags()
    */
   public String[] getRegisteredTags() {
      String[] tags;
      Set s = factoriesMap.keySet();
      tags = new String[s.size()];
      if (!s.isEmpty()) {
         int j = 0;
         for (Iterator i = s.iterator(); i.hasNext(); ) {
            Object item = i.next();
            tags[j] = (String) item;
            j++;
         }
      }

      return tags;

   }
   
   @Override
	public boolean isTagRegistered(String tag) {
		for (Iterator iterator = factoriesMap.keySet().iterator(); iterator.hasNext();) {
			String key = (String) iterator.next();
			if (key.equalsIgnoreCase(tag))
				return true;	
		}
		return false;
	}

}
